@@ -1,4 +1,4 @@
-/*  $Id: BurgerSpaceClient.cpp,v 1.1.2.72 2010/05/16 02:25:47 sarrazip Exp $
+/*  $Id: BurgerSpaceClient.cpp,v 1.3 2010/09/16 03:22:22 sarrazip Exp $
     BurgerSpaceClient.cpp - Game client
 
     burgerspace - A hamburger-smashing video game.
@@ -24,16 +24,19 @@
 
 #include "BurgerSpaceServer.h"
 
+#include <errno.h>
+#include <sys/types.h>
+#include <sys/stat.h>
+
 #include <iostream>
 #include <fstream>
 #include <sstream>
 #include <algorithm>
 
+#ifndef _MSC_VER
 #include <sys/time.h>
-#include <sys/types.h>
-#include <sys/stat.h>
 #include <unistd.h>
-#include <errno.h>
+#endif
 
 #include "pixmaps.h"
 
@@ -129,7 +132,7 @@ BurgerSpaceClient::BurgerSpaceClient(Role _role,
             {
                 theSoundMixer = new SoundMixer(16);  // may throw string
             }
-            catch (const SoundMixer::Error &e)
+            catch (const SoundMixer::Error &)
             {
                 return;
             }
@@ -698,6 +701,9 @@ BurgerSpaceClient::getSavedGamesDir()
     If no such path could be formed, an empty string is returned.
 */
 {
+#ifdef _MSC_VER
+    return "";
+#else
     const char *home = getenv("HOME");  // TODO: what about non-Unix systems?
     if (home == NULL)
         return "";
@@ -738,6 +744,7 @@ BurgerSpaceClient::getSavedGamesDir()
     }
 
     return subdir;
+#endif
 }
 
 
@@ -754,6 +761,7 @@ BurgerSpaceClient::formSavedGameFilename(int slotNum)
 void
 BurgerSpaceClient::showSlotDirectory()
 {
+#ifndef _MSC_VER
     int fontHeight = getFontDimensions().y;
     int y = 11 * fontHeight;
     writeStringXCentered("                           ", y += fontHeight);
@@ -787,6 +795,7 @@ BurgerSpaceClient::showSlotDirectory()
 
     writeStringXCentered("  --  -------------------  ", y += fontHeight);
     writeStringXCentered("                           ", y += fontHeight);
+#endif
 }
 
 
