@@ -1,4 +1,4 @@
-/*  $Id: BurgerSpaceServer.cpp,v 1.3.2.53 2010/05/16 02:25:48 sarrazip Exp $
+/*  $Id: BurgerSpaceServer.cpp,v 1.7 2010/12/31 04:58:16 sarrazip Exp $
     BurgerSpaceServer.cpp - Main engine
 
     burgerspace - A hamburger-smashing video game.
@@ -33,11 +33,14 @@
 
 #include <sys/types.h>
 #include <sys/stat.h>
-#include <unistd.h>
-#include <dirent.h>
 #include <time.h>
 #include <errno.h>
 
+#ifndef _MSC_VER
+#include <unistd.h>
+#include <dirent.h>
+#endif
+
 #include "pixmaps.h"
 
 using namespace std;
@@ -45,11 +48,6 @@ using namespace flatzebra;
 
 #define __(s) (s)  /* Gettext-like macro */
 
-#ifdef _MSC_VER
-        #define VERSION ""
-        #define snprintf _snprintf
-#endif
-
 
 ///////////////////////////////////////////////////////////////////////////////
 //
@@ -857,32 +855,31 @@ BurgerSpaceServer::~BurgerSpaceServer()
 
 
 void
-BurgerSpaceServer::initializeSprites() throw(PixmapLoadError)
-/*  Initializes the sprites that appear at the beginning of a level,
-    like the player and the ingredients.
-
-    Assumes that 'playerSprite' is a valid pointer (may be null).
-
-    Assumes that all pixmap arrays have been loaded.
-*/
+BurgerSpaceServer::createPlayerSprite()
 {
-    /*        Player:
-    */
-    deleteSprite(playerSprite);
+    assert(playerSprite == NULL);
+
     Couple playerSize = playerPA.getImageSize();
     assert(playerSize.isNonZero());
-    int sx = playerStartingPos[theCurrentLevel.getLevelNo()].first;
-    assert(sx != 0);
-    int sy = playerStartingPos[theCurrentLevel.getLevelNo()].second;
-    assert(sy != 0);
-    initPlayerPos = theCurrentLevel.positionInPixels +
-                Couple(sx * TILE_SIDE + 1, sy * TILE_SIDE - playerSize.y);
-    playerSprite = new Sprite(playerPA, initPlayerPos,
+
+    // Init position is bogus: resetPlay() will set it according to current level.
+    playerSprite = new Sprite(playerPA, Couple(0, 0),
                                     Couple(0, 0), Couple(0, 0),
                                     Couple(4, 4), playerSize - Couple(8, 8));
     playerSprite->currentPixmapIndex = size_t(-1);  // invisible at first
+}
 
 
+void
+BurgerSpaceServer::initializeSprites() throw(PixmapLoadError)
+/*  Initializes the sprites that appear at the beginning of a level,
+    like the ingredients.
+
+    Assumes that all pixmap arrays have been loaded.
+
+    DOES NOT create the player sprite.
+*/
+{
     /*        Ingredients:
     */
     deleteSprites(ingredientSprites);
@@ -1062,7 +1059,8 @@ BurgerSpaceServer::update()
             playSoundEffect(newGameStartsSound);
 
             initGameParameters();
-            startNewLife();
+            createPlayerSprite();
+            resetPlay();
             initNextLevel(initLevelNo);
 
             theScore = 0;
@@ -1164,7 +1162,7 @@ BurgerSpaceServer::putSprite(const Sprite &s)
 void
 BurgerSpaceServer::initNextLevel(int levelNo /*= 0*/) throw(int)
 /*  Initialize the next level.  Increments the current level number if
-    levelNo is zero, or uses levelNo is it is positive.
+    levelNo is zero, or uses levelNo if it is positive.
 */
 {
     assert(levelNo >= 0);
@@ -1190,10 +1188,24 @@ BurgerSpaceServer::initNextLevel(int levelNo /*= 0*/) throw(int)
 
 
 void
-BurgerSpaceServer::startNewLife()
+BurgerSpaceServer::resetPlay()
 {
-    deleteSprites(pepperSprites);
+    // Compute the player's initial position in the current level.
+    //
+    Couple playerSize = playerPA.getImageSize();
+    assert(playerSize.isNonZero());
+    int sx = playerStartingPos[theCurrentLevel.getLevelNo()].first;
+    assert(sx != 0);
+    int sy = playerStartingPos[theCurrentLevel.getLevelNo()].second;
+    assert(sy != 0);
+    initPlayerPos = theCurrentLevel.positionInPixels +
+                Couple(sx * TILE_SIDE + 1, sy * TILE_SIDE - playerSize.y);
+
+    assert(playerSprite != NULL);
     playerSprite->setPos(initPlayerPos);
+
+    deleteSprites(pepperSprites);
+
     deleteSprites(enemySprites);
     timeForNewEnemy = 0;
 }
@@ -1294,42 +1306,6 @@ BurgerSpaceServer::chooseDirectionTowardTarget(
 }
 
 
-void
-BurgerSpaceServer::avoidEnemies(bool desiredDirs[4]) const
-/*  Tries to avoids directions that probably lead to a collision
-    with an enemy.
-*/
-{
-    Couple playerPos = playerSprite->getPos();
-
-    SpriteList::const_iterator it;
-    for (it = enemySprites.begin(); it != enemySprites.end(); it++)
-    {
-        const Sprite *enemy = *it;
-
-        // Consider only enemies that are "close" to the player:
-        Couple enemyPos = enemy->getPos();
-        Couple toPlayer = playerPos - enemyPos;
-        if (abs(toPlayer.x) > 2 * TILE_SIDE || abs(toPlayer.y) > 2 * TILE_SIDE)
-            continue;
-
-
-        // What is the direction from this enemy to the player?
-        int dir = chooseDirectionTowardTarget(enemyPos, playerPos,
-                                            PLAYER_SPEED_FACTOR, desiredDirs);
-
-        if (dir == -1)  // if no decision, never mind
-            continue;
-
-        /*  The player must avoid the direction opposite to 'dir'.
-            For example, if the enemy would have to go up to get to the
-            player, then the player must not go down.
-        */
-        desiredDirs[dir ^ 2] = false;
-    }
-}
-
-
 // All pause/resume state changes must be done with a call to this method.
 void
 BurgerSpaceServer::changePauseState(bool newPauseState)
@@ -1343,6 +1319,9 @@ BurgerSpaceServer::animatePlayer()
 /*  Returns true if the game must continue, or false to have it stop.
 */
 {
+    if (playerSprite == NULL)  // if player not created yet
+        return true;  // nothing to do: continue demo mode
+
     /*  If player is agonizing/winning:
     */
     if (playerSprite->getTimeToLive() != 0)
@@ -1379,18 +1358,21 @@ BurgerSpaceServer::animatePlayer()
 
         if (playerSprite->decTimeToLive() == 0)
         {
-            startNewLife();
-
             if (celebrationMode)  // if player just finished the level
             {
-                initNextLevel();  // must come after call to startNewLife()
+                initNextLevel();  // makes the next level the current one
+                resetPlay();  // resets init player pos according to new current level
             }
             else  // player has finished dying
             {
+                resetPlay();  // puts player back into current level's starting pos
                 addToNumLives(-1);
                 assert(numLives >= 0);
                 if (numLives == 0)
-                    playerSprite->currentPixmapIndex = size_t(-1);  // stop displaying chef
+                {
+                    deleteSprite(playerSprite);  // game over 
+                    playerSprite = NULL;
+                }
             }
         }
 
@@ -1679,6 +1661,8 @@ BurgerSpaceServer::spriteBottomCenterIsOverLadder(const Sprite &s) const
 
 void
 BurgerSpaceServer::deleteSprite(Sprite *s) const
+/*  The caller should nullify the pointer after calling this method.
+*/
 {
     if (s != NULL)
     {
@@ -1728,7 +1712,7 @@ BurgerSpaceServer::animateAutomaticCharacters()
 
     /*  Everything freezes if the player is agonizing:
     */
-    if (playerSprite->getTimeToLive() != 0)
+    if (playerSprite != NULL && playerSprite->getTimeToLive() != 0)
         return;
 
 
@@ -2052,8 +2036,6 @@ BurgerSpaceServer::moveEnemyList(SpriteList &slist, int speedFactor)
     to the player.
 */
 {
-    const Couple plpos = playerSprite->getPos();
-
     for (SpriteList::iterator it = slist.begin(); it != slist.end(); it++)
     {
         EnemySprite *s = dynamic_cast<EnemySprite *>(*it);
@@ -2174,9 +2156,9 @@ BurgerSpaceServer::moveEnemyList(SpriteList &slist, int speedFactor)
                 Also, the enemies move randomly if the game is in demo mode.
             */
             int stupidityFactor = rand() % 4;
-            if (stupidityFactor != 0 && numLives != 0)  // if smart mode:
+            if (stupidityFactor != 0 && playerSprite != NULL && numLives != 0)  // if smart mode:
             {
-                dir = chooseDirectionTowardTarget(spos, plpos,
+                dir = chooseDirectionTowardTarget(spos, playerSprite->getPos(),
                                             speedFactor, allowedDirections);
             }
 
@@ -2310,6 +2292,7 @@ BurgerSpaceServer::makePlayerWin()
 {
     playSoundEffect(levelFinishedSound);
 
+    assert(playerSprite != NULL);
     playerSprite->setTimeToLive(80);
     releaseAllCarriedEnemies();
 
@@ -2322,6 +2305,7 @@ BurgerSpaceServer::makePlayerWin()
 void
 BurgerSpaceServer::makePlayerDie()
 {
+    assert(playerSprite != NULL);
     playerSprite->setTimeToLive(60);
         // The player will agonize for this number of ticks.
 
@@ -2347,7 +2331,7 @@ BurgerSpaceServer::releaseAllCarriedEnemies()
 void
 BurgerSpaceServer::detectCollisions()
 {
-    if (playerSprite->getTimeToLive() != 0)  // if player is agonizing
+    if (playerSprite != NULL && playerSprite->getTimeToLive() != 0)  // if player is agonizing
         return;
 
     bool playIngredientLoweredSound = false;
@@ -2372,28 +2356,31 @@ BurgerSpaceServer::detectCollisions()
         {
             assert(aGroup->getNumMembers() > 0);
 
-            for (size_t j = 0; j < aGroup->getNumMembers(); j++)
+            if (playerSprite != NULL)
             {
-                IngredientSprite *ing = aGroup->getMember(j);
-                if (!ing->isLowered() && playerSprite->collidesWithSprite(*ing))
+                for (size_t j = 0; j < aGroup->getNumMembers(); j++)
                 {
-                    playIngredientLoweredSound = true;
-
-                    ing->lower();
-                    if (aGroup->areAllMembersLowered())
+                    IngredientSprite *ing = aGroup->getMember(j);
+                    if (!ing->isLowered() && playerSprite->collidesWithSprite(*ing))
                     {
-                        size_t numCarriedEnemies = carryEnemies(*aGroup);
-                        size_t numFloorsToGo;
-                        switch (numCarriedEnemies)
-                        {
-                            case 0:   numFloorsToGo = 1; break;
-                            case 1:   numFloorsToGo = 3; break;
-                            default:  numFloorsToGo = 5;
-                        }
+                        playIngredientLoweredSound = true;
 
-                        playSoundEffect(ingredientFallsSound);
-                        aGroup->startFalling(
-                                        INGREDIENT_FALL_SPEED, numFloorsToGo);
+                        ing->lower();
+                        if (aGroup->areAllMembersLowered())
+                        {
+                            size_t numCarriedEnemies = carryEnemies(*aGroup);
+                            size_t numFloorsToGo;
+                            switch (numCarriedEnemies)
+                            {
+                                case 0:   numFloorsToGo = 1; break;
+                                case 1:   numFloorsToGo = 3; break;
+                                default:  numFloorsToGo = 5;
+                            }
+
+                            playSoundEffect(ingredientFallsSound);
+                            aGroup->startFalling(
+                                            INGREDIENT_FALL_SPEED, numFloorsToGo);
+                        }
                     }
                 }
             }
@@ -2498,25 +2485,28 @@ BurgerSpaceServer::detectCollisions()
 
     /*  Treats:
     */
-    for (SpriteList::iterator its = treatSprites.begin();
-                                        its != treatSprites.end(); its++)
+    if (playerSprite != NULL)
     {
-        Sprite *treat = *its;
-        assert(treat != NULL);
-        if (treat->collidesWithSprite(*playerSprite))
+        for (SpriteList::iterator its = treatSprites.begin();
+                                            its != treatSprites.end(); its++)
         {
-            long score = (treat->currentPixmapIndex + 1) * 500;
-            createScoreSprites(score, treat->getCenterPos());
+            Sprite *treat = *its;
+            assert(treat != NULL);
+            if (treat->collidesWithSprite(*playerSprite))
+            {
+                long score = (treat->currentPixmapIndex + 1) * 500;
+                createScoreSprites(score, treat->getCenterPos());
 
-            *its = NULL;
-            deleteSprite(treat);
+                *its = NULL;
+                deleteSprite(treat);
 
-            playSoundEffect(chefGetsTreatSound);
-            givePlayerPepper();
+                playSoundEffect(chefGetsTreatSound);
+                givePlayerPepper();
+            }
         }
-    }
 
-    removeNullElementsFromSpriteList(treatSprites);
+        removeNullElementsFromSpriteList(treatSprites);
+    }
 }
 
 
@@ -2583,7 +2573,10 @@ BurgerSpaceServer::detectEnemyCollisions(SpriteList &slist)
             continue;
 
         static bool invincibleChef = (getenv("INVINCIBLECHEF") != NULL);
-        if (!invincibleChef && numLives != 0 && s->collidesWithSprite(*playerSprite))
+        if (!invincibleChef
+                && playerSprite != NULL
+                && numLives != 0
+                && s->collidesWithSprite(*playerSprite))
         {
             playSoundEffect(enemyCatchesChefSound);
             makePlayerDie();
@@ -2916,10 +2909,10 @@ void
 BurgerSpaceServer::drawSprites()
 {
     int oscBit;  // "oscillating" bit used in selecting pixmap array images
-    if (playerSprite->getTimeToLive() == 0)
+    if (playerSprite == NULL || playerSprite->getTimeToLive() == 0)
         oscBit = (tickCount >> 1) & 1;
     else
-        oscBit = 0;
+        oscBit = 0;  // player is agonizing or winning, so freeze everything else
 
 
     /*  Ingredients:
@@ -2946,7 +2939,8 @@ BurgerSpaceServer::drawSprites()
 
     /*  Player:
     */
-    putSprite(*playerSprite);
+    if (playerSprite != NULL && numLives > 0)
+        putSprite(*playerSprite);
 
 
     /*  Peppers:
@@ -3288,6 +3282,9 @@ BurgerSpaceServer::getSavedGamesDir()
     If no such path could be formed, an empty string is returned.
 */
 {
+#ifdef _MSC_VER
+    return "";
+#else
     const char *home = getenv("HOME");  // TODO: what about non-Unix systems?
     if (home == NULL)
         return "";
@@ -3328,6 +3325,7 @@ BurgerSpaceServer::getSavedGamesDir()
     }
 
     return subdir;
+#endif
 }
 
 
